/*
 * Copyright (C) 1998-2006
 * Lehrstuhl fuer Technische Informatik, RWTH-Aachen, Germany
 * 
 * This file is part of the LTI-Computer Vision Library (LTI-Lib)
 *
 * The LTI-Lib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License (LGPL)
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * The LTI-Lib is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with the LTI-Lib; see the file LICENSE.  If
 * not, write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.  
 */ 

#ifndef _LTI_EXAMPLES
#define _LTI_EXAMPLES

/** 

\page examples Examples

Loading and processing an image in a file 

\code
  // Example: Using the ltilib!

  #include <iostream>
  #include <cstdio>
  #include <list>
  #include <cmath>

  #include <ltiMatrix.h>
  #include <ltiChannel.h>
  #include <ltiMath.h>
  #include <ltiIOBMP.h>
  #include <ltiSplitImageTorgI.h>
  #include <ltiConvolution.h>
  #include <ltiDownsampling.h>
  #include <ltiGaussKernels.h>
  #include <ltiSeparableKernel.h>
  #include <ltiViewer2D.h>
        
  using std::cout;
  using std::endl;

  ...

  void testfunction() {  
\endcode

  ------------------------
  simple matrix operations
  ------------------------

\code
    std::cout << "Testing matrix operations and constructor...";

    // data to be used by a matrix
    double mdata[15] = { 1,  2,  3,  4,  5,
                         6,  7,  8,  9, 10,
                        11, 12, 13, 14, 15 };

    // create a matrix "a" and copy the data "mdata" on it.
    lti::matrix<double> a(3,5,mdata);

    // show the matrix
    std::cout << endl << "Matrix a: " << std::endl << a << std::endl;
    
    // add -7 to all elements
    a.add(-7.0);

    // apply the c-function fabs to all elements
    a.apply(fabs);

    // show the resulting matrix
    std::cout << "  After abs(x-7):" << std::endl;

    std::cout << a << std::endl;
    
    // add 0.5 to all elements
    a.add(0.5);
    
    // apply the log function to all elements of a mathVector
    a.apply(log);
    
    // show the matrix after applying "log" to all of its elements
    std::cout << "  After log(x+0.5):" << std::endl << a  << std::endl;

\endcode

  ---------------------
  image procesing tests
  ---------------------

\code
  // create an instance of the image loading functor:
  lti::ioBMP loader;
        
  lti::image img,img2;

  // -- loading an image -- : shortcut
  
  // load the given image using the shortcut!
  loader.load("lenna.png",img);

  // -- extract a monochromatic channel of the image --
  
  // split image test
  
  lti::channel chnl,chnl2;
  
  // functor to split image in the chromaticity color space
  lti::splitImageTorgI splitter;
  
  // extract the intensity channel only!
  splitter.getIntensity(img,chnl);
  
  // -- Convolution and Filters --
  
  // 1D filter tests
  
  cout << "Vector filter...";
  
  lti::vector<float> vct1(65536,0.5);
  lti::vector<float> vct2(65536,0.0f);
  
  // create a gaussian kernel with 5 elements and a variance of 0.7
  lti::gaussKernel1D<float> gauss(5,0.7);
  
  // the convolution functor:
  lti::convolution convolver;
  
  // the parameters for the convolution functor
  lti::convolution::parameters param;
  
  // indicate which kernel should be used
  param.setKernel(gauss);
  
  // set the parameters
  convolver.setParameters(param);
  
  // convolve the vector vct1 with the gaussian kernel
  convolver.apply(vct1,vct2);
  
  // -- 2D separable filters --
  
  // create a (separable) column filter!
  
  std::cout << "Creating separable filter...";
  
  lti::separableKernel<float> sepKern;
  sepKern.setNumberOfPairs(1);
  // in a column filter, the row filter has only 1 element, with value 1
  sepKern.getRowFilter(0).resize(0,0,1,lti::Init);
  // use the gaussian kernel as the column filter
  sepKern.getColFilter(0).copy(gauss);
  
  // indicate that now this column filter should be used
  param.setKernel(sepKern);
  convolver.setParameters(param);
  
  // filter the intensity channel with this column filter
  convolver.apply(chnl,chnl2);
    
  // -- 2D gaussian filter (matrix)
  
  // create a 2D (non-separable) kernel
  lti::kernel2D<float> gauss2D;
  gauss2D.outerProduct(sepKern.getRowFilter(0),sepKern.getColFilter(0));
  
  // indicate the convolution functor to use this kernel
  param.setKernel(gauss2D);
  convolver.setParameters(param);
  
  // convolve the 2D kernel with the channel
  convolver.apply(chnl,chnl2);
  
  // -- downsampling test
  
  // create the downsampling functor (using the default parameters)
  lti::downsampling downsampler;
  
  downsampler.apply(chnl,chnl2);
 
  // create a viewer object to see the images
  lti::viewer2D view;
  
  view.show(chnl2);
  
  getchar(); // wait 'til the user hits a key
  }
\endcode

*/

#endif

