/*
 * Copyright (C) 1998-2006
 * Lehrstuhl fuer Technische Informatik, RWTH-Aachen, Germany
 * 
 * This file is part of the LTI-Computer Vision Library (LTI-Lib)
 *
 * The LTI-Lib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License (LGPL)
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * The LTI-Lib is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with the LTI-Lib; see the file LICENSE.  If
 * not, write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.  
 */ 

/**
 * \file ltiGroups.h
 *
 * This file contains all documentation groups for classes and data structures
 * in the LTI-Lib 2.
 */

#ifndef _LTI_GROUPS
#define _LTI_GROUPS

/**
 * @defgroup gLTILib2FormatRequired Classes imported from the LTI-Lib-1.
 *
 * Please review this classes as soon as possible, since they have
 * still the old LTI-Lib-1 workarounds and format styles (template in
 * the class.h file or similar atrocities) which were required for the old
 * MS Visual compilers but are now not accepted!
 */

/**
 * @defgroup gUnassigned Unassigned Classes and Data Structures
 *
 * Please assign this classes to one of the functionality groups of the
 * LTI-Lib.  If your class does not fit into any of the current groups, please
 * insert a new one in <code>ltilib-2/doc/src/ltiGroups.h</code>
 */

/**
 * @defgroup gDataStruct Data Structures
 *
 * Data Structures in the LTI-Lib
 */

/**
 * @defgroup gTypes Types
 *
 * Types defined in the LTI-Lib
 */


/** 
 * @defgroup gMath Math
 *
 * Classes and Functions related with mathematical operations
 */

/**
 * @defgroup gKernelMath Kernel Math
 *
 * Classes related to kernels and kernel machines, as those used by
 * Support Vector Machines (SVM), Kernel PCA, etc.
 * 
 * @ingroup gMath
 */

/**
 * @defgroup gGeomData Geometry Data Structures
 *
 * Data primitives for geometric algorithms
 * @ingroup gDataStruct
 * @ingroup gGeometry
 */

/**
 * @defgroup gAggregate Aggregate Data Types
 *
 * This group contains aggregate classes, which besides some data also provide 
 * some functionality to manipulate it.
 *
 * @ingroup gDataStruct
 */

/**
 * @defgroup gImageProcessing Image Processing and Analysis
 *
 * This group contains all groups of functors and other classes related
 * with image processing.
 */

/**
 * @defgroup gLinearFilters Linear filtering
 *
 * Linear filtering kernels and functors
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gNonLinearFilters Non-linear filtering
 *
 * Non-linear filtering functors
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gDenoising Denoising filters
 *
 * Denoising filters are non-linear filters used to reduce the amount of noise
 * present in images.
 *
 * @ingroup gNonLinearFilters
 */

/**
 * @defgroup gContrastEnhancement Contrast Enhancement Algorithms
 *
 * Contrast enhancement algorithms improve the use of the available dynamic
 * range in an image.
 *
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gSharpening Sharpening Algorithms
 *
 * Sharpening algorithms improve the definition of the image, in the sense
 * that borders between objects are visually enhanced.
 *
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gEdgeCorner Edge and Corner Detectors
 *
 * Edge and Corner Detectors
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gSegmentation Segmentation Algorithms and related tools
 *
 * Segmentation algorithms and related tools
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gRegionAnalysis Region analysis
 *
 * Algorithms used to generate and used labeled masks denoting regions.
 * @ingroup gSegmentation
 */

/**
 * @defgroup gShape Shape Analysis
 *
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gGeometry Simple geometry classes and functions
 *
 * @ingroup gMath
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gMorphology Morphology operators
 *
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gColor Color Analysis
 *
 * Classes and Data Structures used for color analysis or color representation
 *
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gColorQuantization Color Quantization
 *
 * Classes and Data Structures used for color quantization
 *
 * @ingroup gColor
 */

/**
 * @defgroup gFeatureExtr Descriptor Extraction
 *
 * Classes and Data Structures used to extract descriptors from images
 * or image regions.
 *
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gImageTrans Image Transformations
 *
 * Classes and Data Structures used to transform images between different 
 * domains, or within a domain.
 *
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gTracker Motion Tracking Algorithms
 *
 * Classes that perform motion tracking on image sequences.
 *
 * @ingroup gImageProcessing
 */

/**
 * @defgroup gGeometricTrans Geometric Image Transformations
 *
 * Classes and Data Structures used to transform images geometrically.
 *
 * @ingroup gImageTrans
 */


/**
 * @defgroup gLinearAlgebra Linear Algebra 
 * 
 * @ingroup gMath
 */

/**
 * @defgroup gBasicMathFunc Basic Math Functions
 *
 * @ingroup gMath
 */

/** 
 * @defgroup gRandom Random numbers
 *
 * Classes and Functions related with random number generation.
 *
 * @ingroup gMath
 */

/** 
 * @defgroup gRandomContinuous Continuous random distributions
 *
 * Classes and Functions related with random number generation continuously
 * distributed.
 *
 * @ingroup gRandom
 */

/** 
 * @defgroup gRandomDiscrete Discrete random distributions
 *
 * Classes and Functions related with random number generation of discrete, and
 * finite elements.
 *
 * @ingroup gRandom
 */

/**
 * @defgroup gStatistics Statistics
 *
 * Classes to compute mean values, median, variances, etc.
 *
 * @ingroup gMath
 */

/**
 * @defgroup gOptimization Optimization
 * 
 * Classes use to find optimal values of functions
 *
 * @ingroup gMath
 */

/**
 * @defgroup gIO Input and Output
 *
 * Classes and Functions used to read and write data.
 */

/**
 * @defgroup gStorable Storable Interface
 *
 * Methods using lti::ioHandler objects to read and write data.
 *
 * @ingroup gIO
 */

/**
 * @defgroup gIOImage Image I/O Classes
 *
 * Classes to read and write image files.
 *
 * @ingroup gIO
 */

/**
 * @defgroup gCodecs Encoding and decoding of data.
 *
 * Classes to encode and decode blocks of data.
 *
 * @ingroup gIO
 */


/**
 * @defgroup gVisualization Visualization Tools
 *
 * In this group belong not only the viewer and drawing classes, but all
 * functors used to "paint" data structures on images, in order to simplify
 * their interpretation and analysis.
 */

/**
 * @defgroup gAcquisition Acquisition Tools
 *
 * Mostly cameras and framegrabbers belong in this group. But also
 * classes that coordinate and facilitate the acquisition of images
 * via hardware.
*/

/**
 * @defgroup gLocalFeatures Local Descriptor Processing
 *
 * Classes that deal with the detection, description, classification,
 * and visualization of local descriptors.
 *
 * @ingroup gFeatureExtr
 */ 

/**
 * @defgroup gInterfaces Programming interfaces
 *
 * In this group are located classes that provide some interface, like
 * lti::parametersManager, lti::status, and so on.
 */

/**
 * @defgroup gAlgorithm Classic Algorithms
 *
 * This group holds classical algorithms for searching, sorting, and the like.
 */

/**
 * @defgroup gSorting Sorting and scrambling algorithms
 *
 * This group holds sort algorithms and their opposite ones.
 */

/**
 * @defgroup gSystem System functions
 *
 * This group holds abstractions to system functions
 */
#endif

